/*
 * $Header: /home/projects/mule/scm/mule/mule/src/java/org/mule/components/builder/ReflectionMessageBuilder.java,v 1.1 2005/09/11 18:45:02 rossmason Exp $
 * $Revision: 1.1 $
 * $Date: 2005/09/11 18:45:02 $
 * ------------------------------------------------------------------------------------------------------
 *
 * Copyright (c) SymphonySoft Limited. All rights reserved.
 * http://www.symphonysoft.com
 *
 * The software in this package is published under the terms of the BSD
 * style license a copy of which has been included with this distribution in
 * the LICENSE.txt file.
 */
package org.mule.components.builder;

import org.mule.config.i18n.Message;
import org.mule.config.i18n.Messages;
import org.mule.umo.UMOMessage;
import org.mule.util.ClassHelper;

import java.lang.reflect.Method;
import java.util.List;

/**
 * Will try and set the result of an invocation as a bean property on the request
 * message using reflection
 *
 * @author <a href="mailto:ross.mason@symphonysoft.com">Ross Mason</a>
 * @version $Revision: 1.1 $
 */
public class ReflectionMessageBuilder extends AbstractMessageBuilder {

    public Object buildMessage( UMOMessage request, UMOMessage response) throws MessageBuilderException {
        Object master = request.getPayload();
        Object property = response.getPayload();
        List methods = null;
        try {
            methods = ClassHelper.getSatisfiableMethods(master.getClass(), new Class[]{property.getClass()}, true, true);
        } catch (Exception e) {
            throw new MessageBuilderException(request, e);
        }
        if(methods.size()==0) {
            throw new MessageBuilderException(new Message(Messages.NO_MATCHING_METHODS_FOR_X_ON_X, property.getClass().getName(), master.getClass().getName()), request);
        } else if (methods.size() > 1){
            throw new MessageBuilderException(new Message(Messages.TOO_MANY_MATCHING_METHODS_FOR_X_ON_X, property.getClass().getName(), master.getClass().getName()), request);
        } else {
            Method m = (Method)methods.get(0);
            try {
                m.invoke(master, (property.getClass().isArray() ? (Object[])property : new Object[]{property}));
            } catch (Exception e) {
                throw new MessageBuilderException(new Message(Messages.FAILED_TO_INVOKE_X, m.getName() + " on " + master.getClass().getName()), request);

            }
        }
        return master;
    }
}
