/*
 * $Header: /home/projects/mule/scm/mule/mule/src/java/org/mule/components/script/jsr223/ScriptComponent.java,v 1.1 2005/09/11 18:45:54 rossmason Exp $
 * $Revision: 1.1 $
 * $Date: 2005/09/11 18:45:54 $
 * ------------------------------------------------------------------------------------------------------
 *
 * Copyright (c) SymphonySoft Limited. All rights reserved.
 * http://www.symphonysoft.com
 *
 * The software in this package is published under the terms of the BSD
 * style license a copy of which has been included with this distribution in
 * the LICENSE.txt file.
 */
package org.mule.components.script.jsr223;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.mule.config.i18n.Message;
import org.mule.config.i18n.Messages;
import org.mule.umo.UMOEventContext;
import org.mule.umo.lifecycle.Callable;
import org.mule.umo.lifecycle.Initialisable;
import org.mule.umo.lifecycle.InitialisationException;
import org.mule.umo.lifecycle.RecoverableException;
import org.mule.util.Utility;

import javax.script.*;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;

/**
 * A JSR 223 Script component. Allows any JSR 223 compliant script engines
 * such as javaScript, Groovy or Rhino to be embedded as Mule components
 *
 * @author <a href="mailto:ross.mason@symphonysoft.com">Ross Mason</a>
 * @version $Revision: 1.1 $
 */
public class ScriptComponent implements Callable, Initialisable {

    /**
     * logger used by this class
     */
    protected transient Log logger = LogFactory.getLog(getClass());


    private String scriptText;
    private String scriptFile;

    private CompiledScript compiledScript;
    private ScriptEngine scriptEngine;
    private String scriptEngineName;
    private Namespace namespace;

    public void initialise() throws InitialisationException, RecoverableException {

        if (scriptEngine == null) {
            if (compiledScript == null) {
                if (scriptEngineName != null) {
                    scriptEngine = createScriptEngine();
                }
                if (scriptEngine == null) {
                    throw new InitialisationException(new Message(Messages.PROPERTIES_X_NOT_SET, "scriptEngine, scriptEngineName, compiledScript"), this);
                }
            } else {
                scriptEngine = compiledScript.getEngine();
            }
        }
        namespace = scriptEngine.createNamespace();
        if (compiledScript == null) {
            if (scriptText == null && scriptFile==null) {
                throw new InitialisationException(new Message(Messages.PROPERTIES_X_NOT_SET, "scriptText, scriptFile"), this);
            } else if(scriptText!=null) {
                if (scriptEngine instanceof Compilable) {
                    Compilable compilable = (Compilable) scriptEngine;
                    compileScript(compilable);
                }
            } else {


            }
        }
        if (compiledScript == null) {

        }
    }

    public ScriptEngine getScriptEngine() {
        return scriptEngine;
    }

    public void setScriptEngine(ScriptEngine scriptEngine) {
        this.scriptEngine = scriptEngine;
    }

    public CompiledScript getCompiledScript() {
        return compiledScript;
    }

    public void setCompiledScript(CompiledScript compiledScript) {
        this.compiledScript = compiledScript;
    }

    public String getScriptText() {
        return scriptText;
    }


    public void setScriptText(String scriptText) {
        this.scriptText = scriptText;
    }


    public String getScriptFile() {
        return scriptFile;
    }

    public void setScriptFile(String scriptFile) {
        this.scriptFile = scriptFile;
    }

    public void setScriptEngineName(String scriptEngineName) {
        this.scriptEngineName = scriptEngineName;
    }

    public Namespace getNamespace() {
        return namespace;
    }

    public Object onCall(UMOEventContext eventContext) throws Exception {
        populateNamespace(namespace, eventContext);
        Object result = runScript(namespace);
        if(result==null) {
            result = namespace.get("result");
        }
        return result;
    }

    protected void populateNamespace(Namespace namespace, UMOEventContext context) {
        namespace.put("context", context);
        namespace.put("message", context.getMessage());
        namespace.put("descriptor", context.getComponentDescriptor());
        namespace.put("componentNamespace", this.namespace);
        namespace.put("log", logger);
        namespace.put("result", new Object());
    }

    protected void compileScript(Compilable compilable) throws InitialisationException {

        if (scriptText != null) {
            try {
                compiledScript = compilable.compile(scriptText);
            } catch (ScriptException e) {
                throw new InitialisationException(e, this);
            }
        } else if (scriptFile != null) {
            InputStream is = null;
            try {
                is = Utility.loadResource(scriptFile, getClass());
            } catch (IOException e) {
                throw new InitialisationException(new Message(Messages.CANT_LOAD_X_FROM_CLASSPATH_FILE, scriptFile), e, this);
            }

            try {
                compiledScript = compilable.compile(new InputStreamReader(is));
            } catch (ScriptException e) {
                throw new InitialisationException(e, this);
            }
        }
    }

    protected Object  evaluteScript(Namespace namespace) throws ScriptException {
        return scriptEngine.eval(scriptText, namespace);
    }

    protected Object runScript(Namespace namespace) throws ScriptException {
        Object result = null;
        if (compiledScript != null) {
            result = compiledScript.eval(namespace);
        }
        else {
            result = evaluteScript(namespace);
        }
        return result;
    }

    protected ScriptEngine createScriptEngine()
    {
        ScriptEngineManager manager = new ScriptEngineManager();
        return manager.getEngineByName(scriptEngineName);
    }
}

