/*
 * $Header: /home/projects/mule/scm/mule/mule/src/java/org/mule/impl/InterceptorsInvoker.java,v 1.3 2005/06/03 01:17:24 gnt Exp $
 * $Revision: 1.3 $
 * $Date: 2005/06/03 01:17:24 $
 * ------------------------------------------------------------------------------------------------------
 *
 * Copyright (c) SymphonySoft Limited. All rights reserved.
 * http://www.symphonysoft.com
 *
 * The software in this package is published under the terms of the BSD
 * style license a copy of which has been included with this distribution in
 * the LICENSE.txt file.
 */
package org.mule.impl;

import java.util.List;

import org.mule.umo.Invocation;
import org.mule.umo.UMOException;
import org.mule.umo.UMOImmutableDescriptor;
import org.mule.umo.UMOInterceptor;
import org.mule.umo.UMOMessage;

/**
 * <code>InterceptorsInvoker</code> is used trigger an interceptor chain.
 * 
 * @author <a href="mailto:ross.mason@symphonysoft.com">Ross Mason</a>
 * @version $Revision: 1.3 $
 */

public class InterceptorsInvoker extends Invocation
{
    private List interceptors;
    private int cursor = 0;

    public InterceptorsInvoker(List interceptors, MuleDescriptor descriptor, UMOMessage message)
    {
        this(interceptors, new ImmutableMuleDescriptor(descriptor), message);
    }

    public InterceptorsInvoker(List interceptors, UMOImmutableDescriptor descriptor, UMOMessage message)
    {
        super(descriptor, message, null);
        this.interceptors = interceptors;
    }

    public UMOMessage execute() throws UMOException
    {
        UMOMessage message = null;
        if (cursor < interceptors.size()) {
            UMOInterceptor interceptor = (UMOInterceptor) interceptors.get(cursor);
            incCursor();
            message = interceptor.intercept(this);
            return message;
        }
        return getMessage();
    }

    private synchronized void incCursor()
    {
        cursor++;
    }

}
