/*
 * $Header: /home/projects/mule/scm/mule/mule/src/java/org/mule/impl/internal/events/ManagerEvent.java,v 1.5 2005/06/03 01:17:24 gnt Exp $
 * $Revision: 1.5 $
 * $Date: 2005/06/03 01:17:24 $
 * ------------------------------------------------------------------------------------------------------
 *
 * Copyright (c) SymphonySoft Limited. All rights reserved.
 * http://www.symphonysoft.com
 *
 * The software in this package is published under the terms of the BSD
 * style license a copy of which has been included with this distribution in
 * the LICENSE.txt file.
 */
package org.mule.impl.internal.events;

import org.mule.umo.manager.UMOManager;
import org.mule.umo.manager.UMOServerEvent;

/**
 * <code>ManagerEvent</code> is fired when an event such as the manager
 * starting occurs. The payload of this event will always be a reference to the
 * manager.
 * 
 * @see org.mule.MuleManager
 * @see org.mule.umo.manager.UMOManager
 * @author <a href="mailto:ross.mason@symphonysoft.com">Ross Mason</a>
 * @version $Revision: 1.5 $
 */
public class ManagerEvent extends UMOServerEvent implements BlockingServerEvent
{
    public static final int MANAGER_INITIALISNG = MANAGER_EVENT_ACTION_START_RANGE + 1;
    public static final int MANAGER_INITIALISED = MANAGER_EVENT_ACTION_START_RANGE + 2;
    public static final int MANAGER_STARTING = MANAGER_EVENT_ACTION_START_RANGE + 3;
    public static final int MANAGER_STARTED = MANAGER_EVENT_ACTION_START_RANGE + 4;
    public static final int MANAGER_STOPPING = MANAGER_EVENT_ACTION_START_RANGE + 5;
    public static final int MANAGER_STOPPED = MANAGER_EVENT_ACTION_START_RANGE + 6;
    public static final int MANAGER_DISPOSING = MANAGER_EVENT_ACTION_START_RANGE + 7;
    public static final int MANAGER_DISPOSED = MANAGER_EVENT_ACTION_START_RANGE + 8;
    public static final int MANAGER_DISPOSING_CONNECTORS = MANAGER_EVENT_ACTION_START_RANGE + 9;
    public static final int MANAGER_DISPOSED_CONNECTORS = MANAGER_EVENT_ACTION_START_RANGE + 10;

    private static final transient String[] ACTIONS = new String[] { "initialising", "initialised", "starting",
            "started", "stopping", "stopped", "disposing", "disposed", "disposing connectors", "disposed connectors" };

    public ManagerEvent(UMOManager message, int action)
    {
        super(message, action);
        resourceIdentifier = message.getId();
    }

    protected String getPayloadToString()
    {
        return ((UMOManager) source).getId();
    }

    protected String getActionName(int action)
    {
        int i = action - MANAGER_EVENT_ACTION_START_RANGE;
        if (i - 1 > ACTIONS.length) {
            return String.valueOf(action);
        }
        return ACTIONS[i - 1];
    }
}
