/* 
 * $Header: /home/projects/mule/scm/mule/mule/src/java/org/mule/management/mbeans/EndpointService.java,v 1.3 2005/07/31 18:56:57 rossmason Exp $
 * $Revision: 1.3 $
 * $Date: 2005/07/31 18:56:57 $
 * ------------------------------------------------------------------------------------------------------
 * 
 * Copyright (c) SymphonySoft Limited. All rights reserved.
 * http://www.symphonysoft.com
 * 
 * The software in this package is published under the terms of the BSD
 * style license a copy of which has been included with this distribution in
 * the LICENSE.txt file. 
 *
 */
package org.mule.management.mbeans;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.mule.config.i18n.Message;
import org.mule.config.i18n.Messages;
import org.mule.umo.endpoint.UMOEndpoint;
import org.mule.umo.provider.UMOMessageReceiver;

/**
 * The EndpointServiceMBean allows you to check the confiugration of an endpoint
 * and conect/disconnect endpoints manually.
 * 
 * @author <a href="mailto:ross.mason@symphonysoft.com">Ross Mason</a>
 * @version $Revision: 1.3 $
 */
public class EndpointService implements EndpointServiceMBean
{
    /**
     * logger used by this class
     */
    protected transient Log logger = LogFactory.getLog(getClass());

    private UMOEndpoint endpoint;
    private UMOMessageReceiver receiver;
    private String name;

    public EndpointService(UMOEndpoint endpoint)
    {
        this.endpoint = endpoint;
        init();
    }

    public EndpointService(UMOMessageReceiver receiver)
    {
        if (receiver == null) {
            throw new NullPointerException(new Message(Messages.X_IS_NULL, "Receiver").getMessage());
        }
        this.endpoint = receiver.getEndpoint();
        this.receiver = receiver;
        init();
    }

    private void init()
    {
        if (endpoint == null) {
            throw new NullPointerException(new Message(Messages.X_IS_NULL, "Endpoint").getMessage());
        }
        if (receiver == null && !UMOEndpoint.ENDPOINT_TYPE_RECEIVER.equals(endpoint.getType())) {
            throw new IllegalArgumentException("Recevier is null for Endpoint MBean but the endpoint itself is a receiving endpoint");
        }

        //Endpoint naming is a little messy right now.  Need to tighten it up

        name = endpoint.getConnector().getProtocol().toLowerCase() + "#";
        if(endpoint.getName().startsWith(name)) {
            name = endpoint.getName().replaceAll(":", "#");
        } else {
            name += endpoint.getName().replaceAll(":", "#");
        }

    }

    public String getAddress()
    {
        return endpoint.getEndpointURI().getAddress();
    }

    public String getName()
    {
        return name;
    }

    public boolean isConnected()
    {
        if (receiver == null) {
            return true;
        } else {
            return receiver.isConnected();
        }
    }

    public void connect() throws Exception
    {
        if (receiver != null && !receiver.isConnected()) {
            receiver.connect();
        } else {
            logger.debug("Endpoint is already connected");
        }
    }

    public void disconnect() throws Exception
    {
        if (receiver != null && receiver.isConnected()) {
            receiver.disconnect();
        } else {
            logger.debug("Endpoint is already disconnected");
        }
    }

    public boolean isSynchronous()
    {
        return endpoint.isSynchronous();
    }

    public String getType()
    {
        return endpoint.getType();
    }

}
