/* 
* $Header: /home/projects/mule/scm/mule/providers/email/src/java/org/mule/providers/email/transformers/ObjectToMimeMessage.java,v 1.1 2005/08/24 09:54:12 rossmason Exp $
* $Revision: 1.1 $
* $Date: 2005/08/24 09:54:12 $
* ------------------------------------------------------------------------------------------------------
* 
* Copyright (c) SymphonySoft Limited. All rights reserved.
* http://www.symphonysoft.com
* 
* The software in this package is published under the terms of the BSD
* style license a copy of which has been included with this distribution in
* the LICENSE.txt file. 
*
*/
package org.mule.providers.email.transformers;

import org.mule.transformers.simple.SerializableToByteArray;
import org.mule.umo.UMOEventContext;
import org.mule.umo.transformer.TransformerException;

import javax.activation.DataHandler;
import javax.mail.BodyPart;
import javax.mail.Message;
import javax.mail.MessagingException;
import javax.mail.internet.MimeBodyPart;
import javax.mail.internet.MimeMultipart;
import java.io.IOException;
import java.io.Serializable;
import java.util.Iterator;

/**
 * Transforms a javax.mail.Message to a UMOMEssage and supports attachments
 *
 * @author <a href="mailto:ross.mason@symphonysoft.com">Ross Mason</a>
 * @version $Revision: 1.1 $
 */
public class ObjectToMimeMessage extends StringToEmailMessage {

    protected void setContent(Object payload, Message msg, String contentType, UMOEventContext context) throws Exception {

            if(context.getMessage().getAttachmentNames().size() > 0) {
                MimeMultipart multipart = new MimeMultipart();
                multipart.addBodyPart(getPayloadBodyPart(payload, contentType));
                for (Iterator iterator = context.getMessage().getAttachmentNames().iterator(); iterator.hasNext();) {
                    String name = (String) iterator.next();
                    BodyPart part = getBodyPartForAttachment(context.getMessage().getAttachment(name), name);
                    multipart.addBodyPart(part);
                }

            }
        super.setContent(payload, msg, contentType, context);
    }

    protected BodyPart getBodyPartForAttachment(DataHandler handler, String name) throws MessagingException {
        BodyPart part = new MimeBodyPart();
        part.setDataHandler(handler);
        part.setDescription(name);
        return part;
    }

    protected BodyPart getPayloadBodyPart(Object payload, String contentType) throws MessagingException, TransformerException, IOException {

        DataHandler handler = null;
        if(payload instanceof String) {
            handler = new DataHandler(new PlainTextDataSource(contentType, payload.toString()));
        } else if(payload instanceof Serializable) {
            handler = new DataHandler(new ByteArrayDataSource((byte[])new SerializableToByteArray().transform(payload), contentType));
        }else if(payload instanceof byte[]) {
            handler = new DataHandler(new ByteArrayDataSource((byte[])payload, contentType));
        } else {
            throw new IllegalArgumentException();
        }
        BodyPart part = new MimeBodyPart();
        part.setDataHandler(handler);
        part.setDescription("Payload");
        return part;
    }
}
