/*
 * $Header: /home/projects/mule/scm/mule/providers/jbi/src/java/org/mule/providers/jbi/JbiMessageAdapter.java,v 1.2 2005/07/28 01:37:10 gnt Exp $
 * $Revision: 1.2 $
 * $Date: 2005/07/28 01:37:10 $
 * ------------------------------------------------------------------------------------------------------
 *
 * Copyright (c) SymphonySoft Limited. All rights reserved.
 * http://www.symphonysoft.com
 *
 * The software in this package is published under the terms of the BSD
 * style license a copy of which has been included with this distribution in
 * the LICENSE.txt file.
 */
package org.mule.providers.jbi;

import java.util.Iterator;
import java.util.Set;

import javax.activation.DataHandler;
import javax.jbi.messaging.NormalizedMessage;

import org.mule.providers.AbstractMessageAdapter;
import org.mule.umo.MessagingException;
import org.mule.umo.provider.MessageTypeNotSupportedException;

/**
 * <code>JbiMessageAdapter</code> translates a JBI NormalizedMessage
 * to a UMOMessage
 * 
 * @author <a href="mailto:ross.mason@symphonysoft.com">Ross Mason</a>
 * @version $Revision: 1.2 $
 */

public class JbiMessageAdapter extends AbstractMessageAdapter
{
    private NormalizedMessage message;

    public JbiMessageAdapter(Object message) throws MessagingException
    {
        if (message instanceof NormalizedMessage) {
            this.message = (NormalizedMessage) message;
            for (Iterator iterator = this.message.getPropertyNames().iterator(); iterator.hasNext();) {
                String s = (String) iterator.next();
                properties.put(s, this.message.getProperty(s));
            }
        } else {
            throw new MessageTypeNotSupportedException(message, getClass());
        }
    }


    public void setProperty(Object key, Object value)
    {
        message.setProperty(key.toString(), value);
    }

    public String getPayloadAsString() throws Exception
    {
        return new String();
    }

    public byte[] getPayloadAsBytes() throws Exception
    {
        return new byte[0];
    }

    public Object getPayload()
    {
        return message;
    }

    public Object getProperty(Object key) {
        return message.getProperty(key.toString());
    }

    public void addAttachment(String name, DataHandler dataHandler) throws Exception {
        message.addAttachment(name, dataHandler);
    }

    public void removeAttachment(String name) throws Exception {
        message.removeAttachment(name);
    }

    public DataHandler getAttachment(String name) {
        return message.getAttachment(name);
    }

    public Set getAttachmentNames(){
        return message.getAttachmentNames();
    }

}
