/*
 * $Header: /home/projects/mule/scm/mule/providers/http/src/test/java/org/mule/providers/http/HttpRequestMessageAdapterTestCase.java,v 1.2 2005/07/31 18:56:56 rossmason Exp $
 * $Revision: 1.2 $
 * $Date: 2005/07/31 18:56:56 $
 * ------------------------------------------------------------------------------------------------------
 *
 * Copyright (c) SymphonySoft Limited. All rights reserved.
 * http://www.symphonysoft.com
 *
 * The software in this package is published under the terms of the BSD
 * style license a copy of which has been included with this distribution in
 * the LICENSE.txt file.
 */
package org.mule.providers.http;

import org.mule.providers.http.servlet.HttpRequestMessageAdapter;
import org.mule.tck.providers.AbstractMessageAdapterTestCase;
import org.mule.umo.provider.UMOMessageAdapter;

import javax.servlet.ServletInputStream;
import javax.servlet.http.HttpServletRequest;
import java.io.BufferedReader;
import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.StringReader;
import java.lang.reflect.InvocationHandler;
import java.lang.reflect.Method;
import java.lang.reflect.Proxy;
import java.util.HashMap;
import java.util.Hashtable;
import java.util.Map;

/**
 * @author <a href="mailto:ross.mason@symphonysoft.com">Ross Mason</a>
 * @version $Revision: 1.2 $
 */
public class HttpRequestMessageAdapterTestCase extends AbstractMessageAdapterTestCase {
    public Object getValidMessage() throws Exception {
        return getMockRequest("test message");
    }

    public UMOMessageAdapter createAdapter(Object payload) throws Exception {
        return new HttpRequestMessageAdapter(payload);
    }

    public static HttpServletRequest getMockRequest(final String message) {
        Object proxy = Proxy.newProxyInstance(ServletConnectorTestCase.class.getClassLoader(),
                new Class[]{HttpServletRequest.class},
                new InvocationHandler() {
                    private String payload = message;
                    private Map props = new HashMap();

                    public Object invoke(Object proxy, Method method, Object[] args)
                            throws Throwable {
                        if ("getInputStream".equals(method.getName())) {

                            ServletInputStream s = new ServletInputStream() {
                                ByteArrayInputStream is = new ByteArrayInputStream(payload.getBytes());

                                public int read() throws IOException {
                                    return is.read();
                                }
                            };
                            return s;

                        } else if ("getAttribute".equals(method.getName())) {
                            return props.get(args[0]);
                        } else if ("setAttribute".equals(method.getName())) {
                            props.put(args[0], args[1]);
                        } else if ("equals".equals(method.getName())) {
                            return new Boolean(payload.equals(args[0]));
                        } else if ("toString".equals(method.getName())) {
                            return payload;
                        } else if ("getReader".equals(method.getName())) {
                            return new BufferedReader(new StringReader(payload.toString()));
                        } else if("getAttributeNames".equals(method.getName())) {
                            return new Hashtable().elements();
                        } else if("getHeaderNames".equals(method.getName())) {
                            return new Hashtable().elements();
                        }
                        return null;
                    }
                });
        return (HttpServletRequest) proxy;
    }

    public void testMessageRetrieval() throws Exception {

    }
}
