/*
 * $Header: /home/projects/mule/scm/mule/mule/src/test/java/org/mule/test/util/ExpiryMonitorTestCase.java,v 1.5 2005/06/08 20:52:09 gnt Exp $
 * $Revision: 1.5 $
 * $Date: 2005/06/08 20:52:09 $
 * ------------------------------------------------------------------------------------------------------
 *
 * Copyright (c) SymphonySoft Limited. All rights reserved.
 * http://www.symphonysoft.com
 *
 * The software in this package is published under the terms of the BSD
 * style license a copy of which has been included with this distribution in
 * the LICENSE.txt file.
 */
package org.mule.test.util;

import org.mule.tck.NamedTestCase;
import org.mule.util.monitor.Expirable;
import org.mule.util.monitor.ExpiryMonitor;

/**
 * @author <a href="mailto:ross.mason@symphonysoft.com">Ross Mason</a>
 * @version $Revision: 1.5 $
 */
public class ExpiryMonitorTestCase extends NamedTestCase
{
    private boolean expired = false;

    protected void setUp() throws Exception
    {
        expired = false;
    }

    public void testExpiry() throws InterruptedException
    {
        ExpiryMonitor monitor = new ExpiryMonitor(100);
        Expirable e = new Expirable() {
            public void expired()
            {
                expired = true;
            }
        };
        monitor.addExpirable(300, e);
        Thread.sleep(800);
        assertTrue(expired);
        assertTrue(!monitor.isRegistered(e));
    }

    public void testNotExpiry() throws InterruptedException
    {
        ExpiryMonitor monitor = new ExpiryMonitor(100);
        Expirable e = new Expirable() {
            public void expired()
            {
                expired = true;
            }
        };
        monitor.addExpirable(800, e);
        Thread.sleep(300);
        assertTrue(!expired);
        Thread.sleep(800);
        assertTrue(expired);
        assertTrue(!monitor.isRegistered(e));
    }

    public void testExpiryWithReset() throws InterruptedException
    {
        ExpiryMonitor monitor = new ExpiryMonitor(100);
        Expirable e = new Expirable() {
            public void expired()
            {
                expired = true;
            }
        };
        monitor.addExpirable(600, e);
        Thread.sleep(200);
        assertTrue(!expired);
        monitor.resetExpirable(e);
        Thread.sleep(200);
        assertTrue(!expired);
        Thread.sleep(600);
        assertTrue(expired);

        assertTrue(!monitor.isRegistered(e));
    }

    public void testNotExpiryWithRemove() throws InterruptedException
    {
        ExpiryMonitor monitor = new ExpiryMonitor(100);
        Expirable e = new Expirable() {
            public void expired()
            {
                expired = true;
            }
        };
        monitor.addExpirable(1000, e);
        Thread.sleep(200);
        assertTrue(!expired);
        Thread.sleep(200);
        monitor.removeExpirable(e);
        Thread.sleep(800);
        assertTrue(!expired);
        assertTrue(!monitor.isRegistered(e));
    }
}
